__version__: str = ...
__doc__: str = ...

class WindowManager:
    def __init__(self) -> None: ...
    def start(self, debug: bool = False) -> None: ...
    def start_headless(self, debug: bool = False) -> None: ...
